var getDocs = require('./util/get-docs')
  , registry = require('npm/utils/registry')
  , express = require('express')
  , markdown = require('markdown')
  , pkgcache = {}
  , cacheExp = 1000 * 60 * 60 * 24 // once per day

module.exports = app = express.createServer()

app.configure(function () {
  app.set('view engine', 'jade')
})

//app.use(express.errorHandler({ showStack: true, dumpExceptions: true }))

app.get('/favicon.ico', function (req, res) {
  res.redirect('http://npmjs.org/favicon.ico', 301)
})

// CSS file request.
app.get('/*.css', function(req, res) {
  res.render(req.params[0] + '.sass', { layout: false })
})

app.error(function (er, req, res, next) {
  console.error("%s\t%s", new Date().toISOString(),
                JSON.stringify(req.url))
  if (er.code) res.send(er.code)
  next(er)
})

// Application root/index.
app.get('/', function (req, res, next) {
  registry.get('/', null, cacheExp, false, function (er, data, raw, upstream) {
    if (er) {
      er.code = upstream && upstream.statusCode
      return next(er)
    }
    res.header('expires', new Date(Date.now() + 1000*60*60*24*365).toUTCString())
    res.render('index', {
      locals: {
        title: 'npmdoc'
        , pkgs: data
      }
    })
  })
})

// Package (or package subdirectory) page.
app.get(/^\/([^\/]+)(?:\/([^\/]+))?\/?$/, function (req, res, next) {
  var name = req.params[0]
    , version = req.params[1]

  registry.get(name, undefined, cacheExp, false, function (er, data, raw, upstream) {
    if (er) {
      er.code = upstream && upstream.statusCode
      return next(er)
    }
    if (data['dist-tags'][version]) {
      version = data['dist-tags'][version]
    }
    // If no version specified, get the latest.
    if (!version) {
      version = data['dist-tags'].latest
    }
    var metadata = data
    getDocs(name, version, function (er, data) {
      if (er) return next(er)
      // TODO: Output a list of other versions of the pkg.
      res.header('expires', new Date(Date.now() + 1000*60*60*24*365).toUTCString())
      res.render('pkg', {
        locals: {
          title: name + '@' + version
          , name: name
          , version: version
          , metadata: metadata
          , docpath: undefined
          , markdown: markdown
          , docs: data.docs
          , readme: data.readme
        }
      })
    })
  })
})

// Individual doc page.
app.get(/^\/([^\/]+)(?:\/([^\/]+))\/((?:.+)*)$/, function (req, res, next) {
  var name = req.params[0]
    , version = req.params[1]
    , docpath = req.params[2].split('/').filter(function (elem) {
        return elem.length > 0
      })

  getDocs(name, version, function (er, data) {
    if (er) return next(er)
    if (version !== data.version) {
      // redirect with 302.  Next time, it might be a different location.
      res.header('expires', new Date(Date.now() + 1000*60*60).toUTCString())
      return res.redirect('/'+name+'/'+data.version+'/'+docpath.join('/'), 302)
    }
    var target = data.docs
    // Determine if URL doc path is valid.
    var valid_path = (function () {
      var result = true
      docpath.forEach(function (elem) {
        if (!target[elem]) result = false
        else target = target[elem]
      })
      return result
    })()

    if (!valid_path) {
      var er = new Error("Invalid path")
      er.code = 404
      return next(er)
    }

    // If target is a dir, then display it as such.
    // Otherwise, render the doc.
    res.header('expires', new Date(Date.now() + 1000*60*60*24*365).toUTCString())
    if (typeof target === 'object') {
      res.render('pkg', {
        locals: {
          title: name + '@' + version + '/' + docpath.join('/')
          , name: name
          , version: version
          , metadata: null
          , docpath: docpath.join('/')
          , docs: target
          , readme: undefined
        }
      })
    } else {
      res.render('doc', {
        locals: {
          title: name + '@' + version + '/' + docpath.join('/')
          , name: name
          , version: version
          , docpath: docpath.join('/')
          , markdown: markdown
          , doc: target
        }
      })
    }
  })
})
